var async = require('async');

module.exports = function (data, schema, cb){
  var doc = {};

  function validateLevel (data, schema, property, cb){
    var origin = (property !== null) ? (property + ".") : '';

    async.forEachOf(schema, function (value, key, cb){
      /*
        value: object whit options
        key: property
      */
      var path = origin + key; //Storage the all path: key.key1.key2
      var levels = path.split('.');
      var required = value.required || false;
      var typeError = value.typeError || 'wrong type';
      typeError = ': ' + typeError;
      var requiredError = value.requiredError || 'required';
      requiredError = ': ' + requiredError;
      var prefix = value.prefix || '';
      var pathObject = value.path || null;
      var validators = value.validators || false;

      //Required
      if(typeof data[key] === 'undefined'){ //not exist property
        if(required) return cb(path + requiredError); //Validate if required

        return cb(null);
      }

      //Valiadte data type
      if(({}).toString.call(data[key]).match(/\s([a-z|A-Z]+)/)[1].toLowerCase() !== value.type) return cb(path + typeError);

      //validate regexp
      if(validators){
        for(var i in validators){
          if(! validators[i].validate(data[key])) return cb(path + ': ' + (validators[i].message || 'wrong') );
        }
      }

      var ref = getRef(doc, levels, 1);

      //Find more levels
      if(typeof schema[key].properties !== 'undefined'){
        ref[key] = {};

        validateLevel(data[key], schema[key].properties, path, function (err){
          if(err) return cb(err);

          return cb(null);
        });
      }else{
        //Validate if there are pathObject

        if(pathObject){
          var pathBuild = {}; //Storage transform object
          var next = pathBuild;

          pathObject = pathObject.split('.');

          for(var i in pathObject){
            next[pathObject[i]] = {};
            next = next[pathObject[i]];
          }

          next[key] = data[key]; //Save the original key
          //console.log(next);
          //console.log(pathBuild);

          /*
            Hasta este punto se tiene el pathBuild con el objeto completo
            y el pathObject con los niveles(propiedades) que se utilizaron para contruir el pathBuild
          */

          var relativePath = ref;
          var relativePathBuild = pathBuild;
          var keyNotFound = null;

          for(var i in pathObject){
            if(typeof relativePath[pathObject[i]] == 'undefined'){
              keyNotFound = pathObject[i];
              break;
            } else {
              relativePathBuild = relativePathBuild[pathObject[i]];
              relativePath = relativePath[pathObject[i]];
            }
          }

          if(prefix != '') prefix += '.';

          if(keyNotFound) relativePath[keyNotFound] = relativePathBuild[keyNotFound];
          else relativePath[prefix + key] = data[key];
        }else{
          //validate if there are a prefix
          if(prefix != '') prefix += '.';
          ref[prefix + key] = data[key];
        }

        return cb();
      }
    }, function (err){
      if(err) return cb(err);

      return cb(null);
    });
  }

  validateLevel(data, schema, null, function (err){
    if(err) return cb(err);

    return cb(null, doc);
  });
}

function getRef(obj, levels, less){
  //levels: ['key' ... n]
  less = less || 1;

  var dec = levels.length - less;

  if(dec == 0) return obj;

  for(var i = 0; i < dec; i ++)
    obj = obj[levels[i]];

  return obj;
}
